using System;
using System.Reflection;

namespace Atoms
{
    public class ParOrOp { } // logical class, only for inheritance

    /////////////////////////////////////////////////////

    public enum PType { Open, Close };

    public class Parenthesis : ParOrOp
    {
        public PType GetPType { get; set; }

        public Parenthesis(char c)
        {
            GetPType = (c == '(') ? PType.Open : PType.Close;
        }
    }

    /////////////////////////////////////////////////////

    public enum AType { Val, Var, Fun };
    
    public interface IAtom
    {
        AType GetAType();

        IAtom Copy();
    }

    /////////////////////////////////////////////////////

    // 2.2, 3.1, 0.3 ..etc
    public class Value : IAtom
    {
        public double Content { get; set; }

        public AType GetAType()
        {
            return AType.Val;
        }

        public IAtom Copy()
        {
            return new Value(Content);
        }

        public Value(double d)
        {
            Content = d;
        }

        public Value(string s)
        {
            Content = Double.Parse(s);
        }
    }

    // x, y, z ..etc
    public class Variable : IAtom
    {
        public char Name { get; set; }

        public AType GetAType() 
        {
            return AType.Var;
        }

        public IAtom Copy()
        {
            return new Variable(Name);
        }

        public bool Equals(Variable v)
        {
            return (this.Name == v.Name);
        }

        public Variable(char c)
        {
            Name = c;
        }
    }

    /////////////////////////////////////////////////////

    // allowed functions: Exp, Log, Sqrt, Cos, Sin, Tan, ACos, Asin, Atan, Cosh, Sinh, Tanh (from System.Math ; unary)
    // and: + (Plus), - (Minus), * (Mult), / (Div), ^ (Pow)
    public abstract class Function : ParOrOp, IAtom
    {
        public int Arity { get; set; } // often 1, sometimes 2, could be more... !

        public string Name { get; set; }

        public AType GetAType()
        {
            return AType.Fun;
        }

        public IAtom Copy()
        {
            return CreateFunc(Name);
        }

        public abstract double Eval(params double[] args);

        public static Function CreateFunc(string fname, int a=0) 
        {
            // next replacements are useful to uniformize the treatment of functions (operators are special cases)
            string formattedName = string.Copy(fname);
            if (formattedName.Length == 1)
            {
                switch (formattedName[0])
                {
                    case '+':
                        if (a == 1) formattedName = "uplus";
                        else formattedName = "plus";
                        break;
                    case '-':
                        if (a == 1) formattedName = "uminus";
                        formattedName = "minus";
                        break;
                    case '*':
                        formattedName = "mult";
                        break;
                    case '/':
                        formattedName = "div";
                        break;
                    case '^':
                        formattedName = "pow";
                        break;
                    default: break;
                }
                fname = formattedName.ToLower();
            }
            char firstChar = formattedName[0];
            formattedName = formattedName.Remove(0, 1);
            formattedName = (char)((int)firstChar - 32) + formattedName;
            formattedName = "Atoms." + formattedName; // get the name in the format "Atoms.Mult" ..etc
            Function f = Activator.CreateInstance(null, formattedName).Unwrap() as Function;
            f.Name = fname;
            return f;
        }
    }

}
