using System;
using System.Collections;
using System.Collections.Generic;

using Atoms;
using NTree;
using NExpression;

// functions Tree->List and List->Expression, useful for differentiating
namespace NDecoding
{
    public static class Decode
    {
        // run through the tree to output a list of strings corresponding to RPN
        public static List<IAtom> Tree2List(TreeAtom treeExp)
        {
            var latom = new List<IAtom>();
            T2L_aux(treeExp, ref latom);
            return latom;
        }

        // auxiliary recursive function to run through tree in depth
        public static void T2L_aux(TreeAtom tatom, ref List<IAtom> latom) 
        {
            if (tatom == null) return;
            if (tatom.IsLeaf())
            {
                latom.Add(tatom.Root);
                return;
            }
            int i = 0;
            while (i < tatom.NextGen.Length && tatom.NextGen[i] != null)
            {
                T2L_aux(tatom.NextGen[i++], ref latom);
            }
            latom.Add(tatom.Root);
        }

        public static Expression List2Exp(List<IAtom> listExp)
        {
            // transformation RPN --> ~~ infix "human readable"
            var strsExp = new Stack<string>();
            foreach (var elem in listExp)
            {
                if (elem as Value != null) strsExp.Push(((elem as Value).Content).ToString());
                else if (elem as Variable != null) strsExp.Push(((elem as Variable).Name).ToString());
                else
                {
                    // an operation must be realized (between strings !)
                    Function f = elem as Function;
                    string[] opArgs = new string[f.Arity];
                    for (int i = f.Arity - 1; i >= 0; i--)
                    {
                        opArgs[i] = strsExp.Pop();
                    }
                    string toPrint = f.Name + " ( ";
                    for (int i = 0; i < f.Arity-1; i++) toPrint += opArgs[i] + " , ";
                    toPrint += opArgs[f.Arity-1] + " ) ";
                    strsExp.Push(toPrint);
                }
            }

            string result = strsExp.Pop();
            result = result.Replace("  ", " ");
            // refinement: remove extra parenthesis like in ( x + (y-3) )
            if (result[0] == '(')
            {
                int i = 1;
                int pcounter = 1;
                while (pcounter > 0)
                {
                    if (result[i] == '(') pcounter++;
                    else if (result[i] == ')') pcounter--;
                    i++;
                }
                if (i >= result.Length - 1)
                {
                    result = result.Remove(0, 2);
                    i -= 2;
                    result = result.Remove(i - 2, result.Length - i + 2);
                }
            }
            return new Expression(result);
        }
    }

}
