﻿using System;
using NUnit.Framework;

using NExpression;

namespace EvalExpression.Tests
{
    public class Exo2Tests
    {

        [Test]
        public void Test_ToString()
        {
            var exp11 = new Expression("(4+x)*(1/(1-y))");
            var exp12 = new Expression("( 4 + x) * (1/ (1-y))");
            var exp13 = new Expression("(4 + x)*( 1 / (1-y) )");
            Assert.That(exp11.ToString() == exp12.ToString()
                        && exp12.ToString() == exp13.ToString());

            var exp21 = new Expression("  -  x  ^-3+  [  [ 3.33*y ^ 4 - - a^ (- 2))   / ( 2 + - 3 *u^3 )  * 42 )");
            var exp22 = new Expression(" - x^-3 + ( (3.33* y^4 -  - a^(-2) ]  /  ( 2  + - 3 * u ^ 3 ]  * 42)");
            var exp23 = new Expression("-  x ^-3  +([ 3.33 * y^4 - - a^ (- 2 ) )  /( 2 +- 3* u ^3 )  *42 )");
            Assert.That(exp21.ToString() == exp22.ToString()
                        && exp22.ToString() == exp23.ToString());
        }

        [Test]
        public void Test_Eval() // could be with random values, also..
        {
            var exp1 = new Expression("(4+x) * (1 / (1-y))");
            double val11 = exp1.Eval(1.5, 0.5);
            double val11_ = (4 + 1.5) * (1 / (1 - 0.5)); // 11
            Assert.That(val11 == val11_);
            double val12 = exp1.Eval(-0.1, -2.0);
            double val12_ = (4 + -0.1) * (1 / (1 - - 2.0)); // 1.3
            Assert.That(val12 == val12_);

            var exp2 = new Expression("(2+x) * 0.5 * a^3 - (2*u) / (x-y)");
            double val21 = exp2.Eval(5.0, 10.1, -1.5, -2.0); // a=5 u=10.1 x=-1.5 y=-2
            double val21_ = (2 + -1.5) * 0.5 * Math.Pow(5.0, 3) - (2 * 10.1) / (-1.5 - -2.0);
            Assert.That(val21 == val21_);
            double val22 = exp2.Eval(2.0, 3.1, 3.0, 2.8); // a=2 u=3.1 x=3 y=2.7
            double val22_ = (2 + 3.0) * 0.5 * Math.Pow(2.0, 3) - (2 * 3.1) / (3.0 - 2.8);
            Assert.That(val22 == val22_);
        }

        [Test]
        public void Test_Diff()
        {
            var exp1 = new Expression("(4+x) * (1 / (1-y))");
            Expression dexp1 = exp1.Diff('x');
            double val11 = dexp1.Eval(0.5); // only y remains
            double val11_ = 1 / (1 - 0.5); // 2
            Assert.That(val11 == val11_);
            double val12 = dexp1.Eval(-3.0);
            double val12_ = 1 / (1 - -3.0); // O.25
            Assert.That(val12 == val12_);

            var exp2 = new Expression("(2+x) * 0.5 * a^3 - (2*u) / (x-y)");
            Expression dexp2 = exp2.Diff('x'); // yield 0.5a^3 + (2*u) / (x-y)^2
            double val21 = dexp2.Eval(1.1, 2.3, 1.5, 0.9);
            double val21_ = 0.5 * Math.Pow(1.1, 3) + (2 * 2.3) / Math.Pow(1.5 - 0.9, 2);
            Assert.That(val21 == val21_);
            double val22 = dexp2.Eval(0.6, -2.3, -3.7, -1.4);
            double val22_ = 0.5 * Math.Pow(0.6, 3) + (2 * -2.3) / Math.Pow(-3.7 - -1.4, 2);
            Assert.That(val22 == val22_);
        }
	}
}
